@@ -4,6 +4,8 @@ from typing import Any, Dict, Literal, Optional, TypeVar
 import casadi as cs
 import numpy as np
 import numpy.typing as npt
+from joblib import Memory
+from joblib.memory import MemorizedFunc
 
 from csnlp.core.solutions import Solution, subsevalf
 from csnlp.nlps.constraints import HasConstraints
@@ -11,16 +13,35 @@ from csnlp.nlps.constraints import HasConstraints
 SymType = TypeVar("SymType", cs.SX, cs.MX)
 
 
+def _solve_and_get_stats(
+    solver: MemorizedFunc, kwargs: Dict[str, npt.ArrayLike]
+) -> Dict[str, Any]:
+    """Internal utility to simultaneously run the solver and get its stats."""
+    sol = solver(**kwargs)
+    sol["p"] = kwargs["p"]  # add to solution the parameters for which it was computed
+    sol["stats"] = solver.func.stats()  # add to solution the solver stats
+    return sol
+
+
 class HasObjective(HasConstraints[SymType]):
     """Class for creating an NLP problem with parameters, variables, constraints and an
     objective."""
 
-    __slots__ = ("name", "_f", "_solver", "_solver_opts", "_solver_type", "_failures")
+    __slots__ = (
+        "name",
+        "_f",
+        "_solver",
+        "_solver_opts",
+        "_solver_type",
+        "_cache",
+        "_failures",
+    )
 
     def __init__(
         self,
         sym_type: Literal["SX", "MX"] = "SX",
         remove_redundant_x_bounds: bool = True,
+        cache: Memory = None,
         name: Optional[str] = None,
     ) -> None:
         """Instantiate the class.
@@ -33,12 +54,18 @@ class HasObjective(HasConstraints[SymType]):
             If `True`, then redundant entries in `lbx` and `ubx` are removed when
             properties `h_lbx` and `h_ubx` are called. See these two properties for more
             details. By default, `True`.
+        cache : joblib.Memory, optional
+            Optional cache to avoid computing the same exact NLP more than once. By
+            default, no caching occurs.
+        name : str, optional
+            Name of the NLP scheme. If `None`, it is automatically assigned.
         """
         super().__init__(sym_type, remove_redundant_x_bounds)
         self.name = name
         self._f: Optional[SymType] = None
-        self._solver: Optional[cs.Function] = None
+        self._solver: Optional[MemorizedFunc] = None
         self._solver_opts: Dict[str, Any] = {}
+        self._cache = cache if cache is not None else Memory(None)
         self._failures = 0
 
     @property
@@ -51,7 +78,7 @@ class HasObjective(HasConstraints[SymType]):
     def solver(self) -> Optional[cs.Function]:
         """Gets the NLP optimization solver. Can be `None`, if the solver is not set
         with method `init_solver`."""
-        return self._solver
+        return self._solver.func if self._solver is not None else None
 
     @property
     def solver_opts(self) -> Dict[str, Any]:
@@ -86,11 +113,14 @@ class HasObjective(HasConstraints[SymType]):
         """
         if self._f is None:
             raise RuntimeError("NLP objective not set.")
+
         opts = {} if opts is None else opts.copy()
         con = cs.vertcat(self._g, self._h)
         problem = {"x": self._x, "p": self._p, "g": con, "f": self._f}
         func, stype = (cs.qpsol, "qrqp") if solver == "qp" else (cs.nlpsol, "ipopt")
-        self._solver = func(f"solver_{stype}_{self.name}", stype, problem, opts)
+        solver_func = func(f"solver_{stype}_{self.name}", stype, problem, opts)
+
+        self._solver = self._cache.cache(solver_func)
         self._solver_type = solver
         self._solver_opts = opts
 
@@ -163,10 +193,32 @@ class HasObjective(HasConstraints[SymType]):
             Raises if the solver is un-initialized (see `init_solver`); or if not all
             the parameters are not provided with a numerical value.
         """
-        if pars is None:
-            pars = {}
         if self._solver is None:
             raise RuntimeError("Solver uninitialized.")
+        kwargs = self._process_pars_and_vals0(
+            {
+                "lbx": self._lbx,
+                "ubx": self._ubx,
+                "lbg": np.concatenate((np.zeros(self.ng), np.full(self.nh, -np.inf))),
+                "ubg": 0,
+            },
+            pars,
+            vals0,
+        )
+        sol_with_stats = _solve_and_get_stats(self._solver, kwargs)
+        solution = self._process_solver_sol(sol_with_stats)
+        self._failures += not solution.success
+        return solution
+
+    def _process_pars_and_vals0(
+        self,
+        kwargs: Dict[str, npt.ArrayLike],
+        pars: Optional[Dict[str, npt.ArrayLike]],
+        vals0: Optional[Dict[str, npt.ArrayLike]],
+    ) -> Dict[str, npt.ArrayLike]:
+        """Internal utility to convert pars and initial-val dicts to solver kwargs."""
+        if pars is None:
+            pars = {}
         parsdiff = self._pars.keys() - pars.keys()
         if len(parsdiff) != 0:
             raise RuntimeError(
@@ -174,38 +226,22 @@ class HasObjective(HasConstraints[SymType]):
                 + ", ".join(parsdiff)
                 + "."
             )
-
-        p = subsevalf(self._p, self._pars, pars)
-        kwargs = {
-            "p": p,
-            "lbx": self._lbx,
-            "ubx": self._ubx,
-            "lbg": np.concatenate((np.zeros(self.ng), np.full(self.nh, -np.inf))),
-            "ubg": 0,
-        }
+        kwargs["p"] = subsevalf(self._p, self._pars, pars)
         if vals0 is not None:
             kwargs["x0"] = subsevalf(self._x, self._vars, vals0)
-        sol: Dict[str, cs.DM] = self._solver(**kwargs)
+        return kwargs
 
-        # extract lam_x, lam_g and lam_h
+    def _process_solver_sol(self, sol: Dict[str, Any]) -> Solution:
+        """Internal utility to convert the solver sol dict to a Solution instance."""
         lam_lbx = -cs.fmin(sol["lam_x"], 0)
         lam_ubx = cs.fmax(sol["lam_x"], 0)
         lam_g = sol["lam_g"][: self.ng, :]
         lam_h = sol["lam_g"][self.ng :, :]
-
         vars = self.variables
         vals = {name: subsevalf(var, self._x, sol["x"]) for name, var in vars.items()}
         old = cs.vertcat(
             self._p, self._x, self._lam_g, self._lam_h, self._lam_lbx, self._lam_ubx
         )
-        new = cs.vertcat(p, sol["x"], lam_g, lam_h, lam_lbx, lam_ubx)
+        new = cs.vertcat(sol["p"], sol["x"], lam_g, lam_h, lam_lbx, lam_ubx)
         get_value = partial(subsevalf, old=old, new=new)
-        solution = Solution(
-            f=float(sol["f"]),
-            vars=vars,
-            vals=vals,
-            stats=self._solver.stats().copy(),
-            _get_value=get_value,
-        )
-        self._failures += int(not solution.success)
-        return solution
+        return Solution(float(sol["f"]), vars, vals, sol["stats"], get_value)
