@@ -146,9 +146,7 @@ class NlpScaling(NonRetroactiveWrapper[SymType]):
         simplify: bool = True,
     ) -> Tuple[SymType, ...]:
         """See `Nlp.constraint` method."""
-        return self.nlp.constraint(
-            name, self.unscale(lhs - rhs), op, 0, soft=soft, simplify=simplify
-        )
+        return self.nlp.constraint(name, self.unscale(lhs - rhs), op, 0, soft, simplify)
 
     def minimize(self, objective: SymType) -> None:
         """See `Nlp.minimize` method."""
@@ -175,7 +173,7 @@ class NlpScaling(NonRetroactiveWrapper[SymType]):
             None, Dict[str, npt.ArrayLike], Iterable[Dict[str, npt.ArrayLike]]
         ] = None,
         return_all_sols: bool = False,
-        return_multi_sol: bool = False,
+        return_stacked_sol: bool = False,
     ) -> Union[Solution[SymType], List[Solution[SymType]]]:
         """See `MultistartNlp.solve` method."""
         assert self.nlp.is_multi and hasattr(
@@ -197,7 +195,7 @@ class NlpScaling(NonRetroactiveWrapper[SymType]):
             pars,
             vals0,
             return_all_sols=return_all_sols,
-            return_multi_sol=return_multi_sol,
+            return_stacked_sol=return_stacked_sol,
         )
 
     def _scale_dict(self, d: Dict[str, npt.ArrayLike]) -> Dict[str, npt.ArrayLike]:
