@@ -314,11 +314,7 @@ class NlpSensitivity(Wrapper[SymType]):
         # first order sensitivity, a.k.a., dydp
         Ky = d(self.jacobians["K-y"])
         Kp = d(self.jacobians["K-p"])
-        if solution is None:
-            Ky_inv = cs.inv(Ky)
-            dydp = -Ky_inv @ Kp
-        else:
-            dydp = -np.linalg.solve(Ky, Kp)
+        dydp = (cs.solve if solution is None else np.linalg.solve)(-Ky, Kp)
         if not second_order:
             return dydp, None, None  # type: ignore[return-value]
 
@@ -393,7 +389,7 @@ class NlpSensitivity(Wrapper[SymType]):
             return
 
         sp_J: cs.Sparsity = cs.jacobian(p, self.nlp.p).sparsity()
-        idx = np.asarray(sp_J.get_crs()[1], dtype=int)
+        idx = np.asarray(sp_J.get_crs()[1], int)
         assert idx.size == p.shape[0], (
             "Invalid subset of target parameters (some were not found in the"
             " original NLP parameters)."
@@ -410,7 +406,7 @@ class NlpSensitivity(Wrapper[SymType]):
         # in case of MX, jacobians throw if the MX are indexed (no more
         # symbolical according to the exception). So we take the jacobian
         # with all primal-dual vars, and then index the relevant rows/cols.
-        y = self.nlp.primal_dual_vars(all=True)
+        y = self.nlp.primal_dual_vars(True)
         h_lbx_idx = np.where(self.nlp.lbx != -np.inf)[0]
         h_ubx_idx = np.where(self.nlp.ubx != +np.inf)[0]
         n = self.nlp.nx + self.nlp.ng + self.nlp.nh
